#!/usr/bin/env python
# -*- coding: utf-8 -*-


from gevent import monkey
monkey.patch_all()
import urllib
import urllib2
import sys
import traceback
from cgi import escape


from gevent.pywsgi import WSGIServer
import poster.streaminghttp

opener = poster.streaminghttp.register_openers()

# TODO: create a true configuration file
drop_headers = ['transfer-encoding']

config = {
        'sfd.local:8088':
    {
        'remote_host': 'www.sfdiabete.org',
        'ssl': False
    },
    'linuxfr.local:8088':
    {
        'remote_host': 'linuxfr.org',
    }
}

class RpRedirectHandler(urllib2.HTTPRedirectHandler):

    def http_error_302(self, req, fp, code, msg, headers):
        """ Disable the auto  """
        infourl = urllib.addinfourl(fp, headers, req.get_full_url())
        infourl.code = code
        infourl.msg = msg
        return infourl

class DefaultErrorHandler(urllib2.HTTPDefaultErrorHandler):
    def http_error_default(self, req, fp, code, msg, headers):
        result = urllib2.HTTPError(req.get_full_url(), code, msg, headers, fp)
        result.status = code
        return result


class RequestApp(object):

    def __init__(self, global_config):
        self.env = None
        self.global_config = config
        self.config = None

    def __call__(self, env, start_response):
        """ """
        self.env = env
        local_host = env['HTTP_HOST']
        # TODO: manage auth and url mapping
        if self.global_config.has_key(local_host):
            self.config = self.global_config[local_host]
            return self.on_request(start_response)
        else:
            start_response('404 Not Found', [])
            return []

    def on_request(self, start_response):
        # TODO: manage header and cookie modifications
        print "on_request"
        # Clean this lines
        if self.env.get('QUERY_STRING'):
            url = "http://" + self.config['remote_host'] + self.env['PATH_INFO'] + "?" + self.env['QUERY_STRING']
        else:
            url = "http://" + self.config['remote_host'] + self.env['PATH_INFO']
        print self.env['REQUEST_METHOD'] + "  " + url
        if self.env['REQUEST_METHOD'] == 'POST':
            data = self.env['wsgi.input'].read()
            req = urllib2.Request(url, data,
                    {'Content-Length': self.env['CONTENT_LENGTH'],
                        'Content-Type': self.env['CONTENT_TYPE']})
        else:
            req = urllib2.Request(url)
        # Pass the browser headers to the request
        # TODO: move this in on_req header parser
        for name in (name for name in self.env if name.startswith('HTTP_')):
            value = self.env[name]
            if name == "HTTP_REFERER":
                referer = value.replace(self.env["HTTP_HOST"], self.config['remote_host'])
                req.add_header("Referer", referer)
            if name != "HTTP_HOST" and name != "HTTP_REFERER":
                name = name.split('HTTP_')[1].replace('_', '-')
                req.add_header(name, value)
        try:
            opener = urllib2.build_opener(RpRedirectHandler, DefaultErrorHandler)
            response = opener.open(req)
        except Exception:
            ex = sys.exc_info()[1]
            path = self.env['PATH_INFO']
            sys.stderr.write('error while reading %s:\n' % path)
            traceback.print_exc()
            tb = traceback.format_exc()
            start_response('502 Bad Gateway', [('Content-Type', 'text/html')])
            error_str = escape(str(ex) or ex.__class__.__name__ or 'Error')
            return ['<h1>%s</h1><h2>%s</h2><pre>%s</pre>' % (error_str, escape(path), escape(tb))]
        else:
            data = response.read()
            return self.on_response(start_response, response.code, response.msg, response.headers, data)

    def on_response(self, start_response, code, status, headers, data):
        # TODO; use an response object
        print "on_response"
        for key, value in headers.items():
            if key == 'location':
                # TODO; move this in a default conf
                location = value.replace(self.config['remote_host'], self.env["HTTP_HOST"])
                headers[key] = location
            if key == 'set-cookie':
                # TODO : add cookie parser here
                pass
            if key in drop_headers:
                del headers[key]
        start_response('%d %s' % (code, status), headers.items())
        return [data]


def serve(host,port):
    """Convenience function to immediately start a server instance."""
    s = WSGIServer((host, port), RequestApp(config))
    s.serve_forever()


def _demo():
    serve('localhost', 8088)

if __name__ == "__main__":
    _demo()


